//===========================================================================
// It demonstrates how to get the disassembly lines for all addresses
//===========================================================================

//---------------------------------------------------------------------------

//      ,      .
#define __NT__
#define __IDP__

//---------------------------------------------------------------------------

#include <ida.hpp>
#include <idp.hpp>
#include <bytes.hpp>
#include <loader.hpp>
#include <kernwin.hpp>
#include <expr.hpp>

//--------------------------------------------------------------------------

int init(void) {
  return PLUGIN_OK;
}

//--------------------------------------------------------------------------

void term(void) {
}

//--------------------------------------------------------------------------
void run(int /*arg*/) {
  FILE *fp = fopen("idc.out", "w");
	
  if(fp == NULL) {
    warning("Cannot open idc.out");
    return;
  }
	
	ea_t ea;
	ea_t min_ea = inf.minEA;
	ea_t max_ea = inf.maxEA;
	
	int i, n, flags;
	int j = 0;
	
	for(ea = min_ea; ea < max_ea && ea != BADADDR; ea = next_not_tail(ea)) {  // && ea < min_ea + 100;
		flags = calc_default_idaplace_flags();
		linearray_t ln(&flags);
		idaplace_t pl;
		pl.ea = ea;
		pl.lnnum = 0;
		ln.set_place(&pl);
		
		//msg("Printing disassembly lines:\n");
		
		n = ln.get_linecnt();  // How many lines for this address?
		
    for(i = 0; i < n; i++) {  // Process all of them
      char *line = ln.down();  // Get line
      char buf[MAXSTR];
      
			tag_remove(line, buf, sizeof(buf)); // Remove color codes
      
			// msg("%d: %s\n", i, buf);  // Display it on the message window
			fprintf(fp, "%s\n", buf);
			
			j++;
    }
		
    if(j%1000 == 0)
				msg("%d lines done...\n", j);
  }
	
	msg("\n\n%d lines has written! File 'idc.out' is done.\n", j);
	
  fclose(fp);
}

//--------------------------------------------------------------------------

char comment[] = "Generate disassembly lines for all addresses";
char help[] = "Generate disassembly lines for all addresses\n";

//--------------------------------------------------------------------------

// This is the preferred name of the plugin module in the menu system
// The preferred name may be overriden in plugins.cfg file

char wanted_name[] = "Save the Listing";

//--------------------------------------------------------------------------

// This is the preferred hotkey for the plugin module
// The preferred hotkey may be overriden in plugins.cfg file
// Note: IDA won't tell you if the hotkey is not correct
// It will just disable the hotkey.

char wanted_hotkey[] = "";

//===========================================================================
// PLUGIN DESCRIPTION BLOCK
//===========================================================================
plugin_t PLUGIN =
{
  IDP_INTERFACE_VERSION,
  0,  // Plugin flags
  init,  // Initialize
  term,  // Terminate. this pointer may be NULL.
  run,  // Invoke plugin
  comment,  // Long comment about the plugin it could appear in the status line or as a hint
  help,  // Multiline help about the plugin
  wanted_name,  // The preferred short name of the plugin
  wanted_hotkey  // The preferred hotkey to run the plugin
};

//===========================================================================
//===========================================================================
